﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SearchAndSorting
{
    
    /// <summary>
    /// Сервисный класс 
    /// Реализует методы поиска по образцу
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class Service<T> where T : IComparable<T>
    {
        public delegate int SearchMethod(T[] arr, T pattern);
            
        /// <summary>
        /// Линейный поиск образца в массиве 
        /// </summary>
        /// <param name="massiv">искомый массив</param>
        /// <param name="pattern">образец поиска</param>
        /// <returns>
        /// индекс первого элемента, совпадающего с образцом
        /// или -1, если образец не встречается в массиве
        /// </returns>
        public static int SearchPattern(T[] massiv, T pattern)
        {
            for (int i = 0; i < massiv.Length; i++)
                if (massiv[i].CompareTo(pattern) == 0) return (i);
            return (-1);
        }
        /// <summary>
        /// Вариация линейного поиска образца в массиве 
        /// </summary>
        /// <param name="massiv">искомый массив</param>
        /// <param name="pattern">образец поиска</param>
        /// <returns>
        /// индекс первого элемента, совпадающего с образцом
        /// или -1, если образец не встречается в массиве
        /// </returns>
        public static int SearchPattern1(T[] massiv, T pattern)
        {
            int i = 0;
            while ((i < massiv.Length) && (massiv[i].CompareTo(pattern) != 0))
                i++;
            if (i == massiv.Length) return (-1); else return (i);
        }
        /// <summary>
        /// Линейный поиск с барьером
        /// Предусловие: В массиве существует элемент,
        /// совпадающий с образцом pattern 
        /// </summary>
        /// <param name="massiv">искомый массив</param>
        /// <param name="pattern">образец поиска</param>
        /// <returns>
        /// индекс первого элемента, совпадающего с образцом
        /// </returns>
        public static int SearchBarrier(T[] massiv, T pattern)
        {
            int i = 0;
            while (massiv[i].CompareTo(pattern) != 0)
                i++;
            return (i);
        }

        /// <summary>
        /// Бинарный поиск образца в упорядоченном массиве
        /// Предусловие: Массив упорядочен
        /// </summary>
        /// <param name="massiv">искомый массив</param>
        /// <param name="pattern">образец поиска</param>
        /// <returns>
        /// индекс элемента, совпадающего с образцом,
        /// но не обязательно индекс первого вхождения,
        /// -1, если образец не встречается в массиве
        /// </returns>
        public static int BinSearch(T[] massiv, T pattern)
        {
            int start = 0, finish = massiv.Length - 1, mid = (start + finish) / 2;
            while (start <= finish)
            {
                if (massiv[mid].CompareTo(pattern) == 0) return (mid);
                if (massiv[mid].CompareTo(pattern) == 1)
                    finish = mid - 1;
                else
                    start = mid + 1;
                mid = (start + finish) / 2;
            }
            return (-1);
        }   

        public static long HowLong(SearchMethod search, int count,
            T[] arr, T pattern)
        {
            DateTime start, finish;
            start = DateTime.Now;
            for(int i =0; i < count; i++)
                search(arr,pattern);
            finish = DateTime.Now;
            return finish.Ticks - start.Ticks;
        }

    }
}
